from typing import List

from pydantic import BaseModel


class AiCourse(BaseModel):
    name: str  # 课程名称
    position: str  # 上课地点
    teacher: str  # 上课老师
    weeks: List[int]  # 上课周数
    day: int  # 星期
    start_num: int  # 开始节数
    sections: int  # 连上节数

    @property
    def key(self):
        return f"{','.join([str(i) for i in self.weeks])}_{self.day}_{self.start_num}"

    @property
    def second_key(self):
        return f"{self.day}_{self.sections}"



import cqwu
from cqwu.errors.auth import CookieError
from cqwu.types.epay import PayBillPage


class GetPayBill:
    async def get_pay_bill(
        self: "cqwu.Client",
        page_number: int = 1,
    ) -> PayBillPage:
        """
        获取校园卡消费账单

        Returns:
            PayBillPage: 消费账单
        """
        url = "http://218.194.176.214:8382/epay/thirdapp/bill"
        html = await self.oauth(url)
        if not html:
            raise CookieError()
        if html.url != url:
            raise CookieError()
        data = await self.request.post(
            "http://218.194.176.214:8382/epay/thirdapp/loadbill.json",
            data={"pageno": page_number}
        )
        return PayBillPage(**data.json())



from typing import Optional, Union
from bs4 import BeautifulSoup

import cqwu
from cqwu import types
from cqwu.errors.auth import CookieError


def get_value_from_soup(soup: BeautifulSoup, attr_id: str) -> Union[type(None), str, int]:
    try:
        data = soup.find("input", attrs={"id": attr_id})["value"]
        try:
            return int(data)
        except ValueError:
            return data
    except (ValueError, TypeError, KeyError):
        return None


class GetMe:
    async def get_me(
        self: "cqwu.Client",
    ) -> Optional["types.User"]:
        """
        获取个人信息

        Returns:
            types.User: 个人信息
        """
        url = "http://218.194.176.8/prizepunishnv/studentInfoManageStudentNV!forwardStudentInfo.action"
        html = await self.oauth(url)
        if not html:
            raise CookieError()
        if html.url != url:
            html = await self.request.get(url)
        if html.url != url:
            raise CookieError()
        soup = BeautifulSoup(html.text, "lxml")
        data = {
            "username": "detail_xh",
            "name": "detail_xm",
            "sex": "detail_xb",
            "age": "detail_nl",
            "grade": "detail_nj",
            "institute": "detail_xy",
            "now_class": "detail_bj",
            "join_year": "detail_rxnj",
            "birthday": "detail_csrq",
            "sfz": "detail_sfzh",
            "level": "detail_pycc",
            "home": "detail_hkszd",
        }
        temp = {key: get_value_from_soup(soup, value) for key, value in data.items()}
        temp["password"] = self.password
        try:
            temp["specialty"] = soup.find_all("input", attrs={"id": "detail_xy"})[1]["value"]
        except (ValueError, TypeError, KeyError, IndexError):
            temp["specialty"] = None
        return types.User(**temp)



from typing import Union, List

from bs4 import BeautifulSoup

import cqwu
from cqwu.enums import ExamRound
from cqwu.errors import NoExamData
from cqwu.types import AiExam


class GetExamCalendar:
    async def get_exam_calendar(
        self: "cqwu.Client",
        exam_round: Union[str, ExamRound] = ExamRound.Supplementation,
        xue_nian: int = None,
        xue_qi: int = None,
        use_model: bool = False,
    ) -> Union[str, List[AiExam]]:
        """ 获取考试安排表 """
        xue_nian = xue_nian or self.xue_nian
        xue_qi = xue_qi or self.xue_qi
        exam_round = ExamRound(exam_round)
        jw_html = await self.login_jwmis()
        jw_host = self.get_web_vpn_host(jw_html.url, https=True)
        jw_url = f"{jw_host}/cqwljw/student/ksap.ksapb_date.jsp"
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Language': 'zh-CN,zh;q=0.9,zh-Hans;q=0.8,und;q=0.7,en;q=0.6,zh-Hant;q=0.5,ja;q=0.4',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded',
            'DNT': '1',
            'Pragma': 'no-cache',
            'Referer': f'{jw_host}/cqwljw/student/ksap.ksapb.html?menucode=S20403',
            'Sec-Fetch-Dest': 'iframe',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-User': '?1',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
            'sec-ch-ua': '"Chromium";v="112", "Not:A-Brand";v="99"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }
        data = {
            'xn': str(xue_nian),
            'xq': str(xue_qi),
            'title': '',
            'xnxq': f'{xue_nian}{xue_qi}',
            'kslc': exam_round.value,
        }
        jw_html = await self.request.post(jw_url, data=data, headers=headers, timeout=60, follow_redirects=True)
        if "没有检索到记录!" in jw_html.text:
            raise NoExamData("没有检索到记录!")
        jw_html = jw_html.text.replace("""<script type="text/javascript" src="//clientvpn.cqwu.edu.cn/webvpn/bundle.debug.js" charset="utf-8"></script>""", "")
        jw_html = jw_html.replace("""<script language='javascript' type='text/javascript' src='../js/Print.js'></script>""", "")
        jw_html = jw_html.replace("charset=GBK", 'charset=UTF-8')
        if not use_model:
            return jw_html
        return parse_html(jw_html, exam_round)


def parse_html(html: str, exam_round: ExamRound) -> List[AiExam]:
    data: List[AiExam] = []
    soup = BeautifulSoup(html, "html.parser")
    trs = soup.find_all("tr")[1:]
    for tr in trs:
        tds = tr.find_all("td")
        if len(tds) != 5:
            continue
        data.append(
            AiExam(
                name=tds[0].text,
                credit=float(tds[1].text),
                time=tds[2].text,
                position=tds[3].text,
                seat=tds[4].text,
                exam_round=exam_round,
            )
        )
    return data



from typing import Optional
from bs4 import BeautifulSoup

import cqwu
from cqwu.errors.auth import CookieError


class GetBalance:
    async def get_balance(
        self: "cqwu.Client",
    ) -> Optional[str]:
        """
        获取校园卡余额

        Returns:
            str: 余额
        """
        url = "http://218.194.176.214:8382/epay/thirdapp/balance"
        html = await self.oauth(url)
        if not html:
            raise CookieError()
        if html.url != url:
            raise CookieError()
        soup = BeautifulSoup(html.text, "lxml")
        try:
            return soup.find_all("div", "weui-cell__ft")[2].next
        except (ValueError, TypeError, KeyError, IndexError):
            return ""



from .base import CQWUEhallError


class AuthError(CQWUEhallError):
    pass


class UsernameOrPasswordError(AuthError):
    """ 用户名或密码错误 """


class CookieError(AuthError):
    """ Cookie 失效 """


class NeedCaptchaError(AuthError):
    """ 需要验证码才能登录 """
    def __init__(self, captcha: bytes):
        self.captcha = captcha



import cqwu
from cqwu.errors.auth import CookieError


class LoginWithCookie:
    async def login_with_cookie(
        self: "cqwu.Client",
        cookie: str = None,
    ):
        """
        使用 cookie 登录
        """
        cookie = cookie or self.cookie
        if not cookie:
            raise CookieError()
        self.cookie = cookie  # noqa
        try:
            data = self.cookie.split(";")
            for cookie in data:
                if not cookie:
                    continue
                key, value = cookie.split("=")
                self.cookies.set(key, value)
                self.request.cookies.set(key, value)
            self.me = await self.get_me()  # noqa
        except Exception as e:
            raise CookieError() from e



import cqwu


def get_in_middle(text: str, start: str, end: str) -> str:
    return text.split(start, 1)[1].split(end, 1)[0]


class GetCalendarChange:
    async def get_calendar_change(
        self: "cqwu.Client",
        xue_nian: int = None,
        xue_qi: int = None,
    ) -> str:
        """ 获取课程表 """
        xue_nian = xue_nian or self.xue_nian
        xue_qi = xue_qi or self.xue_qi
        jw_html = await self.login_jwmis()
        jw_host = self.get_web_vpn_host(jw_html.url, https=True)
        jw_url = f"{jw_host}/cqwljw/student/jxap.jxaptzxx_rpt.jsp"
        jw_sg_url = f"{jw_host}/cqwljw/STU_DynamicInitDataAction.do"
        headers = {
            'Accept': '*/*',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Connection': 'keep-alive',
            'Origin': 'https://clientvpn.cqwu.edu.cn',
            'Referer': f'{jw_host}/cqwljw/student/jxap.jxaptzxx.html?menucode=S20302',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.41',
            'content-type': 'application/x-www-form-urlencoded',
            'sec-ch-ua': '"Microsoft Edge";v="111", "Not(A:Brand";v="8", "Chromium";v="111"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }
        params = {
            "classPath": "C73E288D0DEA8D7F772BBD7F8FDC7E66F44C9E3992261989ECBAC5A3D722B306C6354658E0F25121E24CED075326C19885F263F369E5CD668E2EEE7CFB7EB5788F202FC6FD7DB0C96FB6995C1DD96ADE84BE3E72CFFBE9EC74FA044498BD2D21EA0439F9DC625F0EF61B7159924C542D577F814848F27128"
        }
        res = await self.request.post(jw_sg_url, params=params, headers=headers, timeout=60, follow_redirects=True)
        data = {
            'xh': get_in_middle(res.text, '<xh>', '</xh>'),
            'xn': str(xue_nian),
            'xq': str(xue_qi),
            'xnxq': f'{xue_nian},{xue_qi}',
            'menucode_current': 'S20302',
        }
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Cache-Control': 'max-age=0',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Origin': 'https://clientvpn.cqwu.edu.cn',
            'Referer': f'{jw_host}/cqwljw/student/jxap.jxaptzxx.html?menucode=S20302',
            'Sec-Fetch-Dest': 'iframe',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-User': '?1',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.41',
            'sec-ch-ua': '"Microsoft Edge";v="111", "Not(A:Brand";v="8", "Chromium";v="111"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }
        jw_html = await self.request.post(jw_url, data=data, headers=headers, timeout=60, follow_redirects=True)
        jw_html = jw_html.text.replace("""<script type="text/javascript" src="//clientvpn.cqwu.edu.cn/webvpn/bundle.debug.js" charset="utf-8"></script>""", "")
        jw_html = jw_html.replace("""<script language='javascript' type='text/javascript' src='../js/Print.js'></script>""", "")
        return jw_html.replace("<title></title>", '<meta charset="UTF-8">')



class User:
    def __init__(
        self,
        username: int,
        password: str = None,
        name: str = None,
        sex: str = None,
        age: int = None,
        grade: int = None,
        institute: str = None,
        specialty: str = None,
        now_class: str = None,
        join_year: int = None,
        birthday: str = None,
        sfz: str = None,
        level: str = None,
        home: str = None,
    ):
        """
        一个用户

        :param username: 学号
        :param password: 密码
        :param name: 姓名
        :param sex: 性别
        :param age: 年龄
        :param grade: 年级
        :param institute: 学院
        :param specialty: 专业
        :param now_class: 班级
        :param join_year: 入学年份
        :param birthday: 出生日期
        :param sfz: 身份证号
        :param level: 培养层次
        :param home: 户籍地址
        """
        self.username = username
        self.password = password
        self.name = name
        self.sex = sex
        self.age = age
        self.grade = grade
        self.institute = institute
        self.specialty = specialty
        self.now_class = now_class
        self.join_year = join_year
        self.birthday = birthday
        self.sfz = sfz
        self.level = level
        self.home = home



from typing import List

from pydantic import BaseModel


class CP(BaseModel):
    id: int
    name: str
    xue_nian: str
    xue_qi: str
    score: float
    class_rank: int
    class_member: int
    grade_rank: int
    grade_member: int
    dysz: float
    zysz: float
    cxsz: float
    wtsz: float
    zysz_class_rank: int
    zysz_grade_rank: int


class PublicCPRaw(BaseModel):
    XH: str
    XM: str
    DWDM_DISPLAY: str
    ZYDM_DISPLAY: str
    BJDM_DISPLAY: str
    ZCJ: float
    BJPM: int
    ZYNJPM: int
    FS1: float
    FS10: float
    FS11: float
    FS12: float
    DYYSCJ: str
    WTYSCJ: str
    CXYSCJ: str

    @property
    def id(self) -> int:
        """ 学号 """
        return int(self.XH)

    @property
    def name(self) -> str:
        """ 姓名 """
        return self.XM

    @property
    def yuan_xi(self) -> str:
        """ 院系 """
        return self.DWDM_DISPLAY

    @property
    def zhuan_ye(self) -> str:
        """ 专业 """
        return self.ZYDM_DISPLAY

    @property
    def class_name(self) -> str:
        """ 班级 """
        return self.BJDM_DISPLAY

    @property
    def total_score(self) -> float:
        """ 总成绩 """
        return float(self.ZCJ)

    @property
    def class_rank(self) -> int:
        """ 班级排名 """
        return int(self.BJPM)

    @property
    def grade_rank(self) -> int:
        """ 专业年级排名 """
        return int(self.ZYNJPM)

    @property
    def dysz(self) -> float:
        """ 德育素质分 """
        return float(self.FS1)

    @property
    def zysz(self) -> float:
        """ 智育素质测评 """
        return float(self.FS10)

    @property
    def cxsz(self) -> float:
        """ 创新素质测评 """
        return float(self.FS11)

    @property
    def wtsz(self) -> float:
        """ 文体素质 """
        return float(self.FS12)

    @property
    def dysz_raw(self) -> float:
        """ 德育原始成绩 """
        return float(self.DYYSCJ)

    @property
    def wtsz_raw(self) -> float:
        """ 文体原始成绩 """
        return float(self.WTYSCJ)

    @property
    def cxsz_raw(self) -> float:
        """ 创新原始成绩 """
        return float(self.CXYSCJ)


class CPGS(BaseModel):
    totalSize: int
    pageNumber: int
    pageSize: int
    rows: List[PublicCPRaw]



import base64
from typing import Tuple, List, Union

from bs4 import BeautifulSoup

import cqwu
from cqwu.types.calendar import AiCourse


class GetCalendar:
    async def get_calendar(
        self: "cqwu.Client",
        xue_nian: int = None,
        xue_qi: int = None,
        use_model: bool = False,
    ) -> Union[str, List[AiCourse]]:
        """ 获取课程表 """
        xue_nian = xue_nian or self.xue_nian
        xue_qi = xue_qi or self.xue_qi
        jw_html = await self.login_jwmis()
        jw_host = self.get_web_vpn_host(jw_html.url)
        jw_url = f"{jw_host}/cqwljw/student/wsxk.xskcb10319.jsp"
        params = {
            "params": base64.b64encode(f"xn={xue_nian}&xq={xue_qi}".encode()).decode(),
        }
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Connection': 'keep-alive',
            'Referer': f'{jw_host}/cqwljw/student/xkjg.wdkb.jsp?menucode=S20301',
            'Sec-Fetch-Dest': 'iframe',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-User': '?1',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.41',
            'sec-ch-ua': '"Microsoft Edge";v="111", "Not(A:Brand";v="8", "Chromium";v="111"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }
        jw_html = await self.request.get(jw_url, params=params, headers=headers, timeout=60, follow_redirects=True)
        jw_html = jw_html.text.replace("""<script type="text/javascript" src="//clientvpn.cqwu.edu.cn/webvpn/bundle.debug.js" charset="utf-8"></script>""", "")
        return (
            parse_courses(jw_html)
            if use_model
            else jw_html.replace("<title></title>", '<meta charset="UTF-8">')
        )


def parse_courses(jw_html: str) -> List[AiCourse]:
    courses = []
    courses_keys = []
    soup = BeautifulSoup(jw_html, "lxml")
    trs = [i for i in soup.find_all("tr") if i.find("td", {"class": "td"})][:9]
    for tr in trs:
        tds = tr.find_all("td", {"class": "td"})
        for index, td in enumerate(tds):
            divs = td.find_all("div")
            for div in divs:
                class_property = str(div).split("<br/>")
                if len(class_property) != 4:
                    continue
                try:
                    weeks, start_num, sections = parse_weeks_and_sections(
                        BeautifulSoup(class_property[2], "lxml").text.strip()
                    )
                except Exception:
                    continue
                item = AiCourse(
                    name=BeautifulSoup(class_property[0], "lxml").text.strip(),
                    teacher=BeautifulSoup(class_property[1], "lxml").text.strip(),
                    position=BeautifulSoup(class_property[3], "lxml").text.strip(),
                    weeks=weeks,
                    day=index + 1,
                    start_num=start_num,
                    sections=sections,
                )
                if item.key not in courses_keys:
                    courses_keys.append(item.key)
                    courses.append(item)
    return courses


def parse_weeks_and_sections(text: str) -> Tuple[List[int], int, int]:
    # text: 17[1-4],1-18 单 [傍晚1]
    weeks_list, start_num, sections = [], 0, 0
    for week1 in text.split("[")[0].split(","):
        if "-" in week1:
            dan, shuang = "单" in week1, "双" in week1
            week1 = week1.replace("单", "").replace("双", "")
            for week in range(int(week1.split("-")[0]), int(week1.split("-")[1]) + 1):
                if dan:
                    if week % 2 != 0:
                        weeks_list.append(week)
                elif shuang:
                    if week % 2 == 0:
                        weeks_list.append(week)
                else:
                    weeks_list.append(week)
        else:
            weeks_list.append(int(week1))

    section_range = text.split("[")[1].replace("]", "")
    if section_range.startswith("傍晚"):
        start_num = 9
        sections = 1
    elif "-" in section_range:
        start_num = int(section_range.split("-")[0])
        end = int(section_range.split("-")[1])
        sections = end - start_num + 1
    else:
        start_num = int(section_range)
        sections = 1
    return weeks_list, start_num, sections



import cqwu


class LoginWebVPN:
    async def login_web_vpn(
        self: "cqwu.Client",
    ):
        """
        登录 WebVPN
        """
        if not self._use_password_login:
            return
        url = "https://webvpn.cqwu.edu.cn"
        ehall_html = await self.request.get(url, follow_redirects=True)
        self.web_ehall_path = self.get_web_vpn_host(ehall_html.url)  # noqa
        await self.oauth("http://authserver.cqwu.edu.cn/authserver/login?service=https://clientvpn.cqwu.edu.cn/enlink/api/client/callback/cas")
        auth_html = await self.request.get(
            f"{self.web_ehall_path}/login", follow_redirects=True
        )
        if web_auth_path := self.get_web_vpn_host(auth_html.url):
            await self.login_with_password(auth_host=web_auth_path)



from httpx import URL

from .get_calendar import GetCalendar
from .get_calendar_change import GetCalendarChange
from .get_exam_calendar import GetExamCalendar
from .login_jwmis import LoginJwmis
from .login_webvpn import LoginWebVPN


class WebVPN(
    GetCalendar,
    GetCalendarChange,
    GetExamCalendar,
    LoginJwmis,
    LoginWebVPN,
):
    @staticmethod
    def get_web_vpn_host(url: URL, https: bool = False) -> str:
        return next(
            (
                f"https://clientvpn.cqwu.edu.cn/{'https' if https else 'http'}/{i}"
                for i in str(url).split("/")
                if i.startswith("webvpn")
            ),
            None,
        )



from urllib.parse import urlparse

import cqwu


class Oauth:
    async def oauth(
        self: "cqwu.Client",
        url: str,
        host: str = None,
    ):
        """
        使用 统一身份认证平台 登录子系统，并且保存 cookie
        """
        host = host or urlparse(url).hostname
        html = await self.request.get(url, follow_redirects=True)
        return None if html.url.host != host else html



import asyncio
from typing import Coroutine, Optional
from httpx import AsyncClient, Cookies

from cqwu.methods import Methods
from cqwu.types import User


class Client(Methods):
    """CQWU main client."""

    def __init__(
        self,
        username: int = None,
        password: str = None,
        cookie: str = None,
        cookie_file_path: str = "cookie.txt",
        timeout: int = 10,
    ):
        self.username = username
        self.password = password
        self.cookie = cookie
        self.cookie_file_path = cookie_file_path
        self.host = "http://ehall.cqwu.edu.cn"
        self.auth_host = "http://authserver.cqwu.edu.cn"
        self.web_ehall_path = ""
        self.cookies = Cookies()
        self.request = AsyncClient(timeout=timeout)
        self.loop = asyncio.get_event_loop()
        self.me: Optional[User] = None
        self._use_password_login = False
        self.xue_nian = 2022
        """ 学年 """
        self.xue_qi = 1
        """ 学期，0 为第一学期，1 为第二学期 """

    @staticmethod
    def get_input(word: str = "", is_int: bool = False):
        while True:
            value = input(f"请输入{word}：")
            if not value:
                continue
            if is_int:
                try:
                    value = int(value)
                except ValueError:
                    continue
            confirm = (input(f'确认是 "{value}" 吗？(y/N): ')).lower()
            if confirm == "y":
                break
        return value

    def sync(self, coroutine: Coroutine):
        """
        同步执行异步函数

        Args:
            coroutine (Coroutine): 异步函数

        Returns:
            该异步函数的返回值
        """
        return self.loop.run_until_complete(coroutine)



from lxml import etree

import cqwu
from cqwu.errors.auth import UsernameOrPasswordError
from cqwu.utils.auth import encode_password


class LoginWithPassword:
    async def login_with_password(
        self: "cqwu.Client",
        captcha_code: str = None,
        show_qrcode: bool = True,
        auth_host: str = None,
    ):
        """
        使用学号加密码登录
        """
        auth_host = auth_host or self.auth_host
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Cache-Control': 'max-age=0',
            'Connection': 'keep-alive',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.63',
        }
        html = await self.request.get(f"{auth_host}/authserver/login", headers=headers, follow_redirects=True)
        self.cookies.update(html.cookies)
        tree = etree.HTML(html.text)
        try:
            pwd_default_encrypt_salt = tree.xpath('//*[@id="pwdDefaultEncryptSalt"]/@value')[0]
        except IndexError:
            if auth_host == self.auth_host:
                self._use_password_login = True  # noqa
                self.me = await self.get_me()  # noqa
            return
        form_data = {
            'username': str(self.username),
            'password': encode_password(self.password, pwd_default_encrypt_salt),
            'lt': tree.xpath('//*[@id="casLoginForm"]/input[1]/@value')[0],
            'dllt': tree.xpath('//*[@id="casLoginForm"]/input[2]/@value')[0],
            'execution': tree.xpath('//*[@id="casLoginForm"]/input[3]/@value')[0],
            '_eventId': tree.xpath('//*[@id="casLoginForm"]/input[4]/@value')[0],
            'rmShown': tree.xpath('//*[@id="casLoginForm"]/input[5]/@value')[0]
        }
        # 是否需要验证码
        if not captcha_code:
            captcha_code = await self.check_captcha(show_qrcode=show_qrcode)
            if captcha_code:
                form_data['captchaResponse'] = captcha_code
        # 登录
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Cache-Control': 'max-age=0',
            'Connection': 'keep-alive',
            'Origin': auth_host,
            'Referer': f'{auth_host}/authserver/login',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.63',
        }
        html = await self.request.post(
            f"{auth_host}/authserver/login",
            headers=headers,
            data=form_data,
            follow_redirects=False,
        )
        if auth_host == self.auth_host:
            if 'CASTGC' not in html.cookies.keys():
                raise UsernameOrPasswordError
            self.cookies.update(html.cookies)
            self._use_password_login = True  # noqa
            self.me = await self.get_me()  # noqa



from typing import List

import cqwu
from cqwu import types


class GetScore:
    async def get_score(
            self: "cqwu.Client",
            year: int = None,
            semester: int = None,
    ) -> List["types.Score"]:
        """
        获取期末成绩

        Returns:
            List[types.Score]: 成绩列表
        """
        year = year or self.xue_nian
        semester = semester or self.xue_qi
        await self.oauth_xg()
        query_url = "http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/modules/cjcx/cjcxbgdz.do"
        headers = {
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Origin': 'http://xg.cqwu.edu.cn',
            'Referer': 'http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/*default/index.do',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.63',
            'X-Requested-With': 'XMLHttpRequest',
        }
        query_data = [
            {
                "name": "XN",
                "caption": "学年",
                "linkOpt": "AND",
                "builderList": "cbl_m_List",
                "builder": "m_value_equal",
                "value": str(year),
            },
            {
                "name": "XQ",
                "caption": "学期",
                "linkOpt": "AND",
                "builderList": "cbl_m_List",
                "builder": "m_value_equal",
                "value": str(semester),
            }
        ]
        data = {
            'querySetting': json.dumps(query_data),
            'pageSize': '100',
            'pageNumber': '1',
        }
        html = await self.request.post(query_url, headers=headers, data=data)
        data = [types.Score(**i) for i in html.json()["datas"]["cjcxbgdz"]["rows"]]
        return data



import datetime
from typing import Tuple

from pydantic import BaseModel

from cqwu.enums import ExamRound


class AiExam(BaseModel):
    name: str
    """ 课程名称 """
    credit: float
    """ 学分 """
    time: str
    """ 考试时间 """
    position: str
    """ 考试地点 """
    seat: str
    """ 座位号 """
    exam_round: ExamRound
    """ 考试轮次 """

    @property
    def name_no_id(self) -> str:
        """ 获取课程名称(去除课程编号) """
        return self.name.split("]")[-1]

    def get_time(self) -> Tuple[datetime.datetime, datetime.datetime]:
        """ 获取格式化后的考试时间 """
        # 2023-06-25(18周 星期日)09:00-11:00
        day = datetime.datetime.strptime(self.time.split("(")[0], "%Y-%m-%d")
        start_time = datetime.datetime.strptime(self.time.split(")")[1].split("-")[0], "%H:%M")
        start_time = datetime.datetime(day.year, day.month, day.day, start_time.hour, start_time.minute)
        end_time = datetime.datetime.strptime(self.time.split(")")[1].split("-")[1], "%H:%M")
        end_time = datetime.datetime(day.year, day.month, day.day, end_time.hour, end_time.minute)
        return start_time, end_time



from typing import List

from pydantic import BaseModel


class PayBill(BaseModel):
    id: str
    amount: float
    tradename: str
    shopname: str
    paytime: int
    status: int


class PayBillPage(BaseModel):
    pageno: int
    totalpage: int
    retcode: int
    retmsg: str
    dtls: List[PayBill]



from .check_captcha import CheckCaptcha
from .export_cookie_to_file import ExportCookieToFile
from .login import Login
from .login_with_cookie import LoginWithCookie
from .login_with_cookie_file import LoginWithCookieFile
from .login_with_password import LoginWithPassword
from .oauth import Oauth


class Auth(
    CheckCaptcha,
    ExportCookieToFile,
    Login,
    LoginWithCookie,
    LoginWithCookieFile,
    LoginWithPassword,
    Oauth
):
    pass



from os.path import exists

import cqwu
from cqwu.errors.auth import CookieError


class LoginWithCookieFile:
    async def login_with_cookie_file(
        self: "cqwu.Client",
    ):
        """
        使用 cookie 文本文件登录
        """
        if not exists(self.cookie_file_path):
            raise CookieError()

        try:
            with open(self.cookie_file_path, "r") as f:
                self.cookie = f.read()  # noqa
            await self.login_with_cookie()
        except Exception as e:
            raise CookieError() from e



import time

import cqwu
from cqwu.errors.auth import NeedCaptchaError


class CheckCaptcha:
    async def check_captcha(
        self: "cqwu.Client",
        username: int = None,
        show_qrcode: bool = True,
    ):
        """ 检查是否需要验证码 """
        username = username or self.username
        params = {
            "username": username,
            "pwdEncrypt2": "pwdEncryptSalt",
            "_": str(round(time.time() * 1000))
        }
        url = f"{self.auth_host}/authserver/needCaptcha.html"
        captcha_html = await self.request.get(url, params=params, follow_redirects=False)
        if captcha_html.text == 'true':
            params = {
                "ts": str(round(time.time()))
            }
            captcha_url = f"{self.auth_host}/authserver/captcha.html"
            res = await self.request.get(captcha_url, params=params, follow_redirects=False)
            if not show_qrcode:
                raise NeedCaptchaError(res.content)
            with open("captcha.jpg", mode="wb") as f:
                f.write(res.content)
            print("验证码已保存在当前目录下的 captcha.jpg 文件中。")
            return self.get_input("验证码")
        return False



from io import BytesIO

import openpyxl

import cqwu
from cqwu.errors import CookieError
from cqwu.types.cp import CP


class GetCP:
    async def get_cp(
        self: "cqwu.Client",
        year: int = None,
        semester: int = None,
    ) -> CP:
        """ 获取综合测评结果 """
        xue_nian = year or self.xue_nian
        xue_qi = semester or self.xue_qi
        await self.oauth_xg()
        url = "http://xg.cqwu.edu.cn/xsfw/sys/emapcomponent/imexport/export.do"
        headers = {
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6,hu;q=0.5',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Origin': 'http://xg.cqwu.edu.cn',
            'Referer': 'http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/*default/index.do',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.51',
            'X-Requested-With': 'XMLHttpRequest',
        }
        data = {
            'app': 'zhcptybbapp',
            'contextPath': 'http://xg.cqwu.edu.cn/xsfw',
            'module': 'modules',
            'page': 'cpjgcx',
            'action': 'cpjgcxbgdz',
            'containerId': 'cpjg_grid',
            'CPXN': str(xue_nian),
            'CPXQ': str(xue_qi),
            'filename': '综合测评结果',
            'colnames': 'XH,XM,CPXN,CPXQ,DWDM,DZ_ZYFX,BJDM,ZCJ,BJPM,BJRS,ZYNJPM,ZYNJRS,FS1,FS10,FS11,FS12,'
                        'XZNJ,DZ_BJPM,DZ_ZYPM',
        }
        html = await self.request.post(url, headers=headers, data=data)
        if html.status_code != 200:
            raise CookieError()
        html_data = html.json()
        url = f"http://xg.cqwu.edu.cn/xsfw/sys/emapcomponent/file/getAttachmentFile/{html_data['attachment']}.do"
        xlsx_file = await self.request.get(url, headers=headers)
        wb = openpyxl.load_workbook(filename=BytesIO(xlsx_file.content))
        ws = wb.active
        wz = ws[2]
        return CP(
            id=int(wz[0].value),
            name=wz[1].value,
            xue_nian=wz[2].value,
            xue_qi=wz[3].value,
            score=float(wz[7].value),
            class_rank=int(wz[8].value),
            class_member=int(wz[9].value),
            grade_rank=int(wz[10].value),
            grade_member=int(wz[11].value),
            dysz=float(wz[12].value),
            zysz=float(wz[13].value),
            cxsz=float(wz[14].value),
            wtsz=float(wz[15].value),
            zysz_class_rank=int(wz[17].value),
            zysz_grade_rank=int(wz[18].value),
        )



import cqwu
from cqwu.errors.auth import CookieError


class ExportCookieToFile:
    async def export_cookie_to_file(
        self: "cqwu.Client",
    ):
        """
        导出 cookie 到文件
        """
        if not self.cookies:
            raise CookieError()

        data = "".join(f"{key}={value};" for key, value in self.cookies.items())
        with open(self.cookie_file_path, "w") as f:
            f.write(data)



from httpx import Response

import cqwu
from cqwu.errors import CookieError


class LoginJwmis:
    async def login_jwmis(
        self: "cqwu.Client",
    ) -> Response:
        """ 登录教学管理平台 """
        jw_html = await self.request.get(
            f"{self.web_ehall_path}/appShow?appId=5299144291521305", follow_redirects=True
        )
        if "教学管理服务平台" not in jw_html.text:
            raise CookieError
        return jw_html



from typing import List

import cqwu
from cqwu.errors import CookieError
from cqwu.types.cp import CPGS, PublicCPRaw


class GetPublicCP:
    async def get_public_cp(
        self: "cqwu.Client",
        page_size: int = 999,
        page_number: int = 1,
        total: bool = True,
    ) -> List[PublicCPRaw]:
        """ 获取综合测评公示结果 """
        await self.oauth_xg()

        async def get_public_cp_raw(page_size_: int, page_number_: int) -> CPGS:
            url = "http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/modules/cpgs/cpgs_cpgsbg.do"
            headers = {
                'Accept': 'application/json, text/javascript, */*; q=0.01',
                'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6,hu;q=0.5',
                'Connection': 'keep-alive',
                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
                'Origin': 'http://xg.cqwu.edu.cn',
                'Referer': 'http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/*default/index.do',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.51',
                'X-Requested-With': 'XMLHttpRequest',
            }
            data = {
                'querySetting': '[]',
                'pageSize': str(page_size_),
                'pageNumber': str(page_number_),
            }
            html = await self.request.post(url, headers=headers, data=data)
            if html.status_code != 200:
                raise CookieError()
            html_data = html.json()
            html_raw_data = html_data.get("datas", {}).get("cpgs_cpgsbg", {})
            return (
                CPGS(**html_raw_data)
                if html_raw_data
                else CPGS(
                    totalSize=0,
                    pageNumber=page_number_,
                    pageSize=page_size_,
                    rows=[],
                )
            )

        return_datas: List[PublicCPRaw] = []
        if total:
            page_size = 999
            page_number = 1
            while True:
                html_raw_datas = await get_public_cp_raw(page_size, page_number)
                if len(html_raw_datas.rows) == 0:
                    break
                return_datas.extend(html_raw_datas.rows)
                if html_raw_datas.totalSize == 0:
                    break
                elif html_raw_datas.pageNumber * html_raw_datas.pageSize >= html_raw_datas.totalSize:
                    break
                else:
                    page_number += 1
        else:
            html_raw_datas = await get_public_cp_raw(page_size, page_number)
            if len(html_raw_datas.rows) != 0:
                return_datas.extend(html_raw_datas.rows)
        return return_datas



import cqwu

from .get_cp import GetCP
from .get_public_cp import GetPublicCP
from .get_score import GetScore
from ...errors import CookieError


class XG(
    GetCP,
    GetPublicCP,
    GetScore,
):
    async def oauth_xg(
        self: "cqwu.Client",
    ):
        url = "http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/*default/index.do#/cjcx"
        html = await self.oauth(url)
        if not html:
            raise CookieError()
        if html.url != url:
            raise CookieError()
        url = "http://xg.cqwu.edu.cn/xsfw/sys/swpubapp/indexmenu/getAppConfig.do"
        headers = {
            'Accept': '*/*',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6,hu;q=0.5',
            'Connection': 'keep-alive',
            'Referer': 'http://xg.cqwu.edu.cn/xsfw/sys/zhcptybbapp/*default/index.do',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.51',
        }
        params = {
            'appId': '5275772372599202',
            'appName': 'zhcptybbapp',
            'v': '021534151969418724',
        }
        await self.request.get(url, headers=headers, params=params)



from io import BytesIO

import qrcode
from bs4 import BeautifulSoup

import cqwu
from cqwu.errors.auth import CookieError
from cqwu.errors.epay import EPayQrCodeError


class GenPayQrcode:
    async def gen_pay_qrcode(
        self: "cqwu.Client",
        show_qrcode: bool = True,
    ) -> None:
        """
        生成支付二维码
        """
        url = "http://218.194.176.214:8382/epay/thirdconsume/qrcode"
        html = await self.oauth(url)
        if not html:
            raise CookieError()
        if html.url != url:
            raise CookieError()
        soup = BeautifulSoup(html.text, "lxml")
        try:
            data = soup.find("input", attrs={"id": "myText"})["value"]
        except (ValueError, TypeError, KeyError, IndexError):
            return
        qr = qrcode.QRCode()
        qr.add_data(data)
        img = qrcode.make(data)
        if not show_qrcode:
            img_bytes = BytesIO()
            img.save(img_bytes)
            img_bytes.seek(0)
            raise EPayQrCodeError(img_bytes.getvalue())
        qr.print_ascii(invert=True)
        img.save("qrcode.png")
        print("生成支付码到 qrcode.png 成功，请打开该文件查看")



import execjs


encrypt_js = """var CryptoJS=CryptoJS||function(u,p){var d={},l=d.lib={},s=function(){},t=l.Base={extend:function(a){s.prototype=this;var c=new s;a&&c.mixIn(a);c.hasOwnProperty("init")||(c.init=function(){c.$super.init.apply(this,arguments)});c.init.prototype=c;c.$super=this;return c},create:function(){var a=this.extend();a.init.apply(a,arguments);return a},init:function(){},mixIn:function(a){for(var c in a)a.hasOwnProperty(c)&&(this[c]=a[c]);a.hasOwnProperty("toString")&&(this.toString=a.toString)},clone:function(){return this.init.prototype.extend(this)}},
r=l.WordArray=t.extend({init:function(a,c){a=this.words=a||[];this.sigBytes=c!=p?c:4*a.length},toString:function(a){return(a||v).stringify(this)},concat:function(a){var c=this.words,e=a.words,j=this.sigBytes;a=a.sigBytes;this.clamp();if(j%4)for(var k=0;k<a;k++)c[j+k>>>2]|=(e[k>>>2]>>>24-8*(k%4)&255)<<24-8*((j+k)%4);else if(65535<e.length)for(k=0;k<a;k+=4)c[j+k>>>2]=e[k>>>2];else c.push.apply(c,e);this.sigBytes+=a;return this},clamp:function(){var a=this.words,c=this.sigBytes;a[c>>>2]&=4294967295<<
32-8*(c%4);a.length=u.ceil(c/4)},clone:function(){var a=t.clone.call(this);a.words=this.words.slice(0);return a},random:function(a){for(var c=[],e=0;e<a;e+=4)c.push(4294967296*u.random()|0);return new r.init(c,a)}}),w=d.enc={},v=w.Hex={stringify:function(a){var c=a.words;a=a.sigBytes;for(var e=[],j=0;j<a;j++){var k=c[j>>>2]>>>24-8*(j%4)&255;e.push((k>>>4).toString(16));e.push((k&15).toString(16))}return e.join("")},parse:function(a){for(var c=a.length,e=[],j=0;j<c;j+=2)e[j>>>3]|=parseInt(a.substr(j,
2),16)<<24-4*(j%8);return new r.init(e,c/2)}},b=w.Latin1={stringify:function(a){var c=a.words;a=a.sigBytes;for(var e=[],j=0;j<a;j++)e.push(String.fromCharCode(c[j>>>2]>>>24-8*(j%4)&255));return e.join("")},parse:function(a){for(var c=a.length,e=[],j=0;j<c;j++)e[j>>>2]|=(a.charCodeAt(j)&255)<<24-8*(j%4);return new r.init(e,c)}},x=w.Utf8={stringify:function(a){try{return decodeURIComponent(escape(b.stringify(a)))}catch(c){throw Error("Malformed UTF-8 data");}},parse:function(a){return b.parse(unescape(encodeURIComponent(a)))}},
q=l.BufferedBlockAlgorithm=t.extend({reset:function(){this._data=new r.init;this._nDataBytes=0},_append:function(a){"string"==typeof a&&(a=x.parse(a));this._data.concat(a);this._nDataBytes+=a.sigBytes},_process:function(a){var c=this._data,e=c.words,j=c.sigBytes,k=this.blockSize,b=j/(4*k),b=a?u.ceil(b):u.max((b|0)-this._minBufferSize,0);a=b*k;j=u.min(4*a,j);if(a){for(var q=0;q<a;q+=k)this._doProcessBlock(e,q);q=e.splice(0,a);c.sigBytes-=j}return new r.init(q,j)},clone:function(){var a=t.clone.call(this);
a._data=this._data.clone();return a},_minBufferSize:0});l.Hasher=q.extend({cfg:t.extend(),init:function(a){this.cfg=this.cfg.extend(a);this.reset()},reset:function(){q.reset.call(this);this._doReset()},update:function(a){this._append(a);this._process();return this},finalize:function(a){a&&this._append(a);return this._doFinalize()},blockSize:16,_createHelper:function(a){return function(b,e){return(new a.init(e)).finalize(b)}},_createHmacHelper:function(a){return function(b,e){return(new n.HMAC.init(a,
e)).finalize(b)}}});var n=d.algo={};return d}(Math);
(function(){var u=CryptoJS,p=u.lib.WordArray;u.enc.Base64={stringify:function(d){var l=d.words,p=d.sigBytes,t=this._map;d.clamp();d=[];for(var r=0;r<p;r+=3)for(var w=(l[r>>>2]>>>24-8*(r%4)&255)<<16|(l[r+1>>>2]>>>24-8*((r+1)%4)&255)<<8|l[r+2>>>2]>>>24-8*((r+2)%4)&255,v=0;4>v&&r+0.75*v<p;v++)d.push(t.charAt(w>>>6*(3-v)&63));if(l=t.charAt(64))for(;d.length%4;)d.push(l);return d.join("")},parse:function(d){var l=d.length,s=this._map,t=s.charAt(64);t&&(t=d.indexOf(t),-1!=t&&(l=t));for(var t=[],r=0,w=0;w<
l;w++)if(w%4){var v=s.indexOf(d.charAt(w-1))<<2*(w%4),b=s.indexOf(d.charAt(w))>>>6-2*(w%4);t[r>>>2]|=(v|b)<<24-8*(r%4);r++}return p.create(t,r)},_map:"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="}})();
(function(u){function p(b,n,a,c,e,j,k){b=b+(n&a|~n&c)+e+k;return(b<<j|b>>>32-j)+n}function d(b,n,a,c,e,j,k){b=b+(n&c|a&~c)+e+k;return(b<<j|b>>>32-j)+n}function l(b,n,a,c,e,j,k){b=b+(n^a^c)+e+k;return(b<<j|b>>>32-j)+n}function s(b,n,a,c,e,j,k){b=b+(a^(n|~c))+e+k;return(b<<j|b>>>32-j)+n}for(var t=CryptoJS,r=t.lib,w=r.WordArray,v=r.Hasher,r=t.algo,b=[],x=0;64>x;x++)b[x]=4294967296*u.abs(u.sin(x+1))|0;r=r.MD5=v.extend({_doReset:function(){this._hash=new w.init([1732584193,4023233417,2562383102,271733878])},
_doProcessBlock:function(q,n){for(var a=0;16>a;a++){var c=n+a,e=q[c];q[c]=(e<<8|e>>>24)&16711935|(e<<24|e>>>8)&4278255360}var a=this._hash.words,c=q[n+0],e=q[n+1],j=q[n+2],k=q[n+3],z=q[n+4],r=q[n+5],t=q[n+6],w=q[n+7],v=q[n+8],A=q[n+9],B=q[n+10],C=q[n+11],u=q[n+12],D=q[n+13],E=q[n+14],x=q[n+15],f=a[0],m=a[1],g=a[2],h=a[3],f=p(f,m,g,h,c,7,b[0]),h=p(h,f,m,g,e,12,b[1]),g=p(g,h,f,m,j,17,b[2]),m=p(m,g,h,f,k,22,b[3]),f=p(f,m,g,h,z,7,b[4]),h=p(h,f,m,g,r,12,b[5]),g=p(g,h,f,m,t,17,b[6]),m=p(m,g,h,f,w,22,b[7]),
f=p(f,m,g,h,v,7,b[8]),h=p(h,f,m,g,A,12,b[9]),g=p(g,h,f,m,B,17,b[10]),m=p(m,g,h,f,C,22,b[11]),f=p(f,m,g,h,u,7,b[12]),h=p(h,f,m,g,D,12,b[13]),g=p(g,h,f,m,E,17,b[14]),m=p(m,g,h,f,x,22,b[15]),f=d(f,m,g,h,e,5,b[16]),h=d(h,f,m,g,t,9,b[17]),g=d(g,h,f,m,C,14,b[18]),m=d(m,g,h,f,c,20,b[19]),f=d(f,m,g,h,r,5,b[20]),h=d(h,f,m,g,B,9,b[21]),g=d(g,h,f,m,x,14,b[22]),m=d(m,g,h,f,z,20,b[23]),f=d(f,m,g,h,A,5,b[24]),h=d(h,f,m,g,E,9,b[25]),g=d(g,h,f,m,k,14,b[26]),m=d(m,g,h,f,v,20,b[27]),f=d(f,m,g,h,D,5,b[28]),h=d(h,f,
m,g,j,9,b[29]),g=d(g,h,f,m,w,14,b[30]),m=d(m,g,h,f,u,20,b[31]),f=l(f,m,g,h,r,4,b[32]),h=l(h,f,m,g,v,11,b[33]),g=l(g,h,f,m,C,16,b[34]),m=l(m,g,h,f,E,23,b[35]),f=l(f,m,g,h,e,4,b[36]),h=l(h,f,m,g,z,11,b[37]),g=l(g,h,f,m,w,16,b[38]),m=l(m,g,h,f,B,23,b[39]),f=l(f,m,g,h,D,4,b[40]),h=l(h,f,m,g,c,11,b[41]),g=l(g,h,f,m,k,16,b[42]),m=l(m,g,h,f,t,23,b[43]),f=l(f,m,g,h,A,4,b[44]),h=l(h,f,m,g,u,11,b[45]),g=l(g,h,f,m,x,16,b[46]),m=l(m,g,h,f,j,23,b[47]),f=s(f,m,g,h,c,6,b[48]),h=s(h,f,m,g,w,10,b[49]),g=s(g,h,f,m,
E,15,b[50]),m=s(m,g,h,f,r,21,b[51]),f=s(f,m,g,h,u,6,b[52]),h=s(h,f,m,g,k,10,b[53]),g=s(g,h,f,m,B,15,b[54]),m=s(m,g,h,f,e,21,b[55]),f=s(f,m,g,h,v,6,b[56]),h=s(h,f,m,g,x,10,b[57]),g=s(g,h,f,m,t,15,b[58]),m=s(m,g,h,f,D,21,b[59]),f=s(f,m,g,h,z,6,b[60]),h=s(h,f,m,g,C,10,b[61]),g=s(g,h,f,m,j,15,b[62]),m=s(m,g,h,f,A,21,b[63]);a[0]=a[0]+f|0;a[1]=a[1]+m|0;a[2]=a[2]+g|0;a[3]=a[3]+h|0},_doFinalize:function(){var b=this._data,n=b.words,a=8*this._nDataBytes,c=8*b.sigBytes;n[c>>>5]|=128<<24-c%32;var e=u.floor(a/
4294967296);n[(c+64>>>9<<4)+15]=(e<<8|e>>>24)&16711935|(e<<24|e>>>8)&4278255360;n[(c+64>>>9<<4)+14]=(a<<8|a>>>24)&16711935|(a<<24|a>>>8)&4278255360;b.sigBytes=4*(n.length+1);this._process();b=this._hash;n=b.words;for(a=0;4>a;a++)c=n[a],n[a]=(c<<8|c>>>24)&16711935|(c<<24|c>>>8)&4278255360;return b},clone:function(){var b=v.clone.call(this);b._hash=this._hash.clone();return b}});t.MD5=v._createHelper(r);t.HmacMD5=v._createHmacHelper(r)})(Math);
(function(){var u=CryptoJS,p=u.lib,d=p.Base,l=p.WordArray,p=u.algo,s=p.EvpKDF=d.extend({cfg:d.extend({keySize:4,hasher:p.MD5,iterations:1}),init:function(d){this.cfg=this.cfg.extend(d)},compute:function(d,r){for(var p=this.cfg,s=p.hasher.create(),b=l.create(),u=b.words,q=p.keySize,p=p.iterations;u.length<q;){n&&s.update(n);var n=s.update(d).finalize(r);s.reset();for(var a=1;a<p;a++)n=s.finalize(n),s.reset();b.concat(n)}b.sigBytes=4*q;return b}});u.EvpKDF=function(d,l,p){return s.create(p).compute(d,
l)}})();
CryptoJS.lib.Cipher||function(u){var p=CryptoJS,d=p.lib,l=d.Base,s=d.WordArray,t=d.BufferedBlockAlgorithm,r=p.enc.Base64,w=p.algo.EvpKDF,v=d.Cipher=t.extend({cfg:l.extend(),createEncryptor:function(e,a){return this.create(this._ENC_XFORM_MODE,e,a)},createDecryptor:function(e,a){return this.create(this._DEC_XFORM_MODE,e,a)},init:function(e,a,b){this.cfg=this.cfg.extend(b);this._xformMode=e;this._key=a;this.reset()},reset:function(){t.reset.call(this);this._doReset()},process:function(e){this._append(e);return this._process()},
finalize:function(e){e&&this._append(e);return this._doFinalize()},keySize:4,ivSize:4,_ENC_XFORM_MODE:1,_DEC_XFORM_MODE:2,_createHelper:function(e){return{encrypt:function(b,k,d){return("string"==typeof k?c:a).encrypt(e,b,k,d)},decrypt:function(b,k,d){return("string"==typeof k?c:a).decrypt(e,b,k,d)}}}});d.StreamCipher=v.extend({_doFinalize:function(){return this._process(!0)},blockSize:1});var b=p.mode={},x=function(e,a,b){var c=this._iv;c?this._iv=u:c=this._prevBlock;for(var d=0;d<b;d++)e[a+d]^=
c[d]},q=(d.BlockCipherMode=l.extend({createEncryptor:function(e,a){return this.Encryptor.create(e,a)},createDecryptor:function(e,a){return this.Decryptor.create(e,a)},init:function(e,a){this._cipher=e;this._iv=a}})).extend();q.Encryptor=q.extend({processBlock:function(e,a){var b=this._cipher,c=b.blockSize;x.call(this,e,a,c);b.encryptBlock(e,a);this._prevBlock=e.slice(a,a+c)}});q.Decryptor=q.extend({processBlock:function(e,a){var b=this._cipher,c=b.blockSize,d=e.slice(a,a+c);b.decryptBlock(e,a);x.call(this,
e,a,c);this._prevBlock=d}});b=b.CBC=q;q=(p.pad={}).Pkcs7={pad:function(a,b){for(var c=4*b,c=c-a.sigBytes%c,d=c<<24|c<<16|c<<8|c,l=[],n=0;n<c;n+=4)l.push(d);c=s.create(l,c);a.concat(c)},unpad:function(a){a.sigBytes-=a.words[a.sigBytes-1>>>2]&255}};d.BlockCipher=v.extend({cfg:v.cfg.extend({mode:b,padding:q}),reset:function(){v.reset.call(this);var a=this.cfg,b=a.iv,a=a.mode;if(this._xformMode==this._ENC_XFORM_MODE)var c=a.createEncryptor;else c=a.createDecryptor,this._minBufferSize=1;this._mode=c.call(a,
this,b&&b.words)},_doProcessBlock:function(a,b){this._mode.processBlock(a,b)},_doFinalize:function(){var a=this.cfg.padding;if(this._xformMode==this._ENC_XFORM_MODE){a.pad(this._data,this.blockSize);var b=this._process(!0)}else b=this._process(!0),a.unpad(b);return b},blockSize:4});var n=d.CipherParams=l.extend({init:function(a){this.mixIn(a)},toString:function(a){return(a||this.formatter).stringify(this)}}),b=(p.format={}).OpenSSL={stringify:function(a){var b=a.ciphertext;a=a.salt;return(a?s.create([1398893684,
1701076831]).concat(a).concat(b):b).toString(r)},parse:function(a){a=r.parse(a);var b=a.words;if(1398893684==b[0]&&1701076831==b[1]){var c=s.create(b.slice(2,4));b.splice(0,4);a.sigBytes-=16}return n.create({ciphertext:a,salt:c})}},a=d.SerializableCipher=l.extend({cfg:l.extend({format:b}),encrypt:function(a,b,c,d){d=this.cfg.extend(d);var l=a.createEncryptor(c,d);b=l.finalize(b);l=l.cfg;return n.create({ciphertext:b,key:c,iv:l.iv,algorithm:a,mode:l.mode,padding:l.padding,blockSize:a.blockSize,formatter:d.format})},
decrypt:function(a,b,c,d){d=this.cfg.extend(d);b=this._parse(b,d.format);return a.createDecryptor(c,d).finalize(b.ciphertext)},_parse:function(a,b){return"string"==typeof a?b.parse(a,this):a}}),p=(p.kdf={}).OpenSSL={execute:function(a,b,c,d){d||(d=s.random(8));a=w.create({keySize:b+c}).compute(a,d);c=s.create(a.words.slice(b),4*c);a.sigBytes=4*b;return n.create({key:a,iv:c,salt:d})}},c=d.PasswordBasedCipher=a.extend({cfg:a.cfg.extend({kdf:p}),encrypt:function(b,c,d,l){l=this.cfg.extend(l);d=l.kdf.execute(d,
b.keySize,b.ivSize);l.iv=d.iv;b=a.encrypt.call(this,b,c,d.key,l);b.mixIn(d);return b},decrypt:function(b,c,d,l){l=this.cfg.extend(l);c=this._parse(c,l.format);d=l.kdf.execute(d,b.keySize,b.ivSize,c.salt);l.iv=d.iv;return a.decrypt.call(this,b,c,d.key,l)}})}();
(function(){for(var u=CryptoJS,p=u.lib.BlockCipher,d=u.algo,l=[],s=[],t=[],r=[],w=[],v=[],b=[],x=[],q=[],n=[],a=[],c=0;256>c;c++)a[c]=128>c?c<<1:c<<1^283;for(var e=0,j=0,c=0;256>c;c++){var k=j^j<<1^j<<2^j<<3^j<<4,k=k>>>8^k&255^99;l[e]=k;s[k]=e;var z=a[e],F=a[z],G=a[F],y=257*a[k]^16843008*k;t[e]=y<<24|y>>>8;r[e]=y<<16|y>>>16;w[e]=y<<8|y>>>24;v[e]=y;y=16843009*G^65537*F^257*z^16843008*e;b[k]=y<<24|y>>>8;x[k]=y<<16|y>>>16;q[k]=y<<8|y>>>24;n[k]=y;e?(e=z^a[a[a[G^z]]],j^=a[a[j]]):e=j=1}var H=[0,1,2,4,8,
16,32,64,128,27,54],d=d.AES=p.extend({_doReset:function(){for(var a=this._key,c=a.words,d=a.sigBytes/4,a=4*((this._nRounds=d+6)+1),e=this._keySchedule=[],j=0;j<a;j++)if(j<d)e[j]=c[j];else{var k=e[j-1];j%d?6<d&&4==j%d&&(k=l[k>>>24]<<24|l[k>>>16&255]<<16|l[k>>>8&255]<<8|l[k&255]):(k=k<<8|k>>>24,k=l[k>>>24]<<24|l[k>>>16&255]<<16|l[k>>>8&255]<<8|l[k&255],k^=H[j/d|0]<<24);e[j]=e[j-d]^k}c=this._invKeySchedule=[];for(d=0;d<a;d++)j=a-d,k=d%4?e[j]:e[j-4],c[d]=4>d||4>=j?k:b[l[k>>>24]]^x[l[k>>>16&255]]^q[l[k>>>
8&255]]^n[l[k&255]]},encryptBlock:function(a,b){this._doCryptBlock(a,b,this._keySchedule,t,r,w,v,l)},decryptBlock:function(a,c){var d=a[c+1];a[c+1]=a[c+3];a[c+3]=d;this._doCryptBlock(a,c,this._invKeySchedule,b,x,q,n,s);d=a[c+1];a[c+1]=a[c+3];a[c+3]=d},_doCryptBlock:function(a,b,c,d,e,j,l,f){for(var m=this._nRounds,g=a[b]^c[0],h=a[b+1]^c[1],k=a[b+2]^c[2],n=a[b+3]^c[3],p=4,r=1;r<m;r++)var q=d[g>>>24]^e[h>>>16&255]^j[k>>>8&255]^l[n&255]^c[p++],s=d[h>>>24]^e[k>>>16&255]^j[n>>>8&255]^l[g&255]^c[p++],t=
d[k>>>24]^e[n>>>16&255]^j[g>>>8&255]^l[h&255]^c[p++],n=d[n>>>24]^e[g>>>16&255]^j[h>>>8&255]^l[k&255]^c[p++],g=q,h=s,k=t;q=(f[g>>>24]<<24|f[h>>>16&255]<<16|f[k>>>8&255]<<8|f[n&255])^c[p++];s=(f[h>>>24]<<24|f[k>>>16&255]<<16|f[n>>>8&255]<<8|f[g&255])^c[p++];t=(f[k>>>24]<<24|f[n>>>16&255]<<16|f[g>>>8&255]<<8|f[h&255])^c[p++];n=(f[n>>>24]<<24|f[g>>>16&255]<<16|f[h>>>8&255]<<8|f[k&255])^c[p++];a[b]=q;a[b+1]=s;a[b+2]=t;a[b+3]=n},keySize:8});u.AES=p._createHelper(d)})();function _gas(data,key0,
iv0){key0=key0.replace(/(^\s+)|(\s+$)/g, "");var key  = CryptoJS.enc.Utf8.parse(key0);var iv   = CryptoJS.enc.Utf8.parse(iv0);var encrypted =CryptoJS.AES.encrypt(data,key,{iv:iv,mode:CryptoJS.mode.CBC,padding:CryptoJS.pad.Pkcs7});return encrypted.toString();}function encryptAES(data,_p1){if(!_p1){return data;}var encrypted =_gas(_rds(64)+data,_p1,_rds(16));return encrypted;}function _ep(p0,p1) {try{return encryptAES(p0,p1);}catch(e){}return p0;
}var $_chars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678';var _chars_len = $_chars.length;function _rds(len) {var retStr = '';for (i = 0; i < len; i++) {retStr += $_chars.charAt(Math.floor(Math.random() * _chars_len));}return retStr;};"""


def encode_password(password: str, pwd_default_encrypt_salt: str) -> str:
    ctx = execjs.compile(encrypt_js)
    return ctx.call("encryptAES", password, pwd_default_encrypt_salt)



from pydantic import BaseModel


class Score(BaseModel):
    """ 成绩类 """
    KCMC: str
    XF: float
    ZCJ: float
    JD: str
    XN: str
    XN_DISPLAY: str
    XQ: str
    XQ_DISPLAY: str

    @property
    def name(self) -> str:
        """ 课程名称 """
        return self.KCMC

    @property
    def credit(self) -> float:
        """ 学分 """
        return self.XF

    @property
    def score(self) -> float:
        """ 成绩 """
        return self.ZCJ

    @property
    def grade_point(self) -> float:
        """ 绩点 """
        return float(self.JD)

    @property
    def year(self) -> int:
        """ 学年 """
        return int(self.XN)

    @property
    def semester(self) -> int:
        """ 学期 """
        return int(self.XQ)



import contextlib
from os.path import exists

import cqwu
from cqwu.errors.auth import CookieError


class Login:
    async def login(
        self: "cqwu.Client",
    ):
        """ 登录 """
        with contextlib.suppress(CookieError):
            if self.cookie:
                await self.login_with_cookie()
            elif exists(self.cookie_file_path):
                await self.login_with_cookie_file()
            return
        if self.username and self.password:
            await self.login_with_password()
        else:
            raise CookieError()
